<?php
/**
 * Class SB_Instagram_Feed_Pro
 *
 * The Pro class mostly adds additional methods
 * used in the "display_instagram" function for supporting
 * additional features.
 *
 * @since 5.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	die( '-1' );
}

class SB_Instagram_Feed_Pro extends SB_Instagram_Feed
{
	/**
	 * @var array
	 */
	private $username_avatars;

	/**
	 * the API_Connect class can use either a premade url or
	 * settings from a connected account, type, and parameters
	 *
	 * @param array|string $connected_account_or_page
	 * @param null $type
	 * @param null $params
	 *
	 * @return object|SB_Instagram_API_Connect_Pro
	 */
	public function make_api_connection( $connected_account_or_page, $type = NULL, $params = NULL ) {
		return new SB_Instagram_API_Connect_Pro( $connected_account_or_page, $type, $params );
	}

	/**
	 * Used to trigger the appending of recent hashtag posts
	 * that are only available in the custom database tables
	 *
	 * @return bool
	 */
	public function out_of_next_pages() {
		return $this->get_next_pages() === false;
	}

	/**
	 * Only recent hashtag feeds need posts from the db currently
	 * but this could change in the future
	 *
	 * @param array $settings
	 * @param array $feed_type_and_terms
	 *
	 * @return bool
	 */
	public function should_look_for_db_only_posts( $settings, $feed_type_and_terms ) {
		return isset( $feed_type_and_terms['hashtags_recent'] );
	}

	/**
	 * Recent hashtag feeds need older posts added from the database
	 * as they aren't available from the API after 24 hours
	 *
	 * @param string $feed_id
	 * @param array $settings
	 * @param array $feed_type_and_terms
	 */
	public function add_db_only_posts( $feed_id, $settings, $feed_type_and_terms ) {
		$feed_id_array = explode( '#', $feed_id );
		$query_feed_id = $feed_id_array[0];
		$post_data = $this->get_post_data();
		$last_post_data_set = array_slice( $post_data, -$settings['minnum'], $settings['minnum'] );

		$earliest_time_stamp = date( 'Y-m-d H:i:s' );
		if ( isset( $last_post_data_set[0] ) ) {
			// usort($last_post_data_set, 'sbi_date_sort' ); not accurate, instead sort all posts by placeholder time stamp after
			// looping through the last post set. this is only necessary from randomly sorted hashtag posts, should be rare

			$last_post = array_slice( $last_post_data_set, -1 );

			$earliest_time_stamp = SB_Instagram_Feed_Pro::get_time_stamp( $feed_id, $last_post[0]['id']);
		}

		$post_set = SB_Instagram_Feed_Pro::get_post_set_from_db( $query_feed_id, 0, $earliest_time_stamp );

		if ( isset( $post_set[0] ) ) {

			foreach ( $post_set as $post ) {
				$decoded = json_decode( $post, true );
				if ( isset( $decoded['id'] ) ) {
					$decoded = $this->filter_posts( array( $decoded ), $settings );

					if ( ! empty( $decoded[0] ) ) {
						array_push( $post_data, $decoded[0] );
					}
				}
			}

			$this->set_post_data( $post_data );
		}

	}

	/**
	 * Queries the database for post data. Useful for getting posts no longer available in the API
	 * or getting them more efficiently.
	 *
	 * @param string $feed_id
	 * @param int $offset
	 * @param string $max_timestamp formatted date
	 * @param int $num_posts
	 *
	 * @return array
	 */
	public static function get_post_set_from_db( $feed_id, $offset, $max_timestamp, $num_posts = 200 ) {
		global $wpdb;

		$posts_table_name = $wpdb->prefix . SBI_INSTAGRAM_POSTS_TYPE;
		$feeds_posts_table_name = $wpdb->prefix . SBI_INSTAGRAM_FEEDS_POSTS;

		$results = $wpdb->get_col( $wpdb->prepare( "SELECT p.json_data FROM $posts_table_name AS p INNER JOIN $feeds_posts_table_name AS f ON p.id = f.id 
			WHERE BINARY f.feed_id = %s
			AND p.time_stamp < '$max_timestamp'
			ORDER BY p.time_stamp
			DESC LIMIT %d, %d", $feed_id, $offset, $num_posts ) );

		return $results;
	}

	public static function get_time_stamp( $feed_id, $post_id ) {
		global $wpdb;

		$posts_table_name = $wpdb->prefix . SBI_INSTAGRAM_POSTS_TYPE;
		$feeds_posts_table_name = $wpdb->prefix . SBI_INSTAGRAM_FEEDS_POSTS;
		$feed_id_array = explode( '#', $feed_id );
		$feed_id = $feed_id_array[0];
		$is_top_post_feed = (substr( $feed_id, 4, 1 ) === '+');

		$timestamp_column = $is_top_post_feed ? 'top_time_stamp' : 'time_stamp';

		$results = $wpdb->get_col( $wpdb->prepare( "SELECT p.$timestamp_column FROM $posts_table_name AS p INNER JOIN $feeds_posts_table_name AS f ON p.id = f.id 
			WHERE f.instagram_id = %s 
			ORDER BY p.$timestamp_column
			ASC LIMIT 1", $post_id ) );

		$return = isset( $results[0] ) ? $results[0] : date( 'Y-m-d H:i:s' );

		return $return;
	}

	/**
	 * Connects to the Instagram API to retrieve the current "story"
	 *
	 * @param $settings
	 * @param array $feed_types_and_terms
	 * @param array $connected_accounts_for_feed
	 *
	 * @since 5.0
	 */
	public function set_remote_stories_data( $settings, $feed_types_and_terms, $connected_accounts_for_feed ) {
		$first_user = isset( $feed_types_and_terms['users'][0] ) ? $feed_types_and_terms['users'][0]['term'] : '';
		$header_data = $this->get_header_data();

		if ( isset( $connected_accounts_for_feed[ $first_user ] ) && isset( $connected_accounts_for_feed[ $first_user ]['type'] ) && $connected_accounts_for_feed[ $first_user ]['type'] === 'business' ) {
			$connection = new SB_Instagram_API_Connect_Pro( $connected_accounts_for_feed[ $first_user ], 'stories', array() );

			$connection->connect();

			if ( ! $connection->is_wp_error() && ! $connection->is_instagram_error() ) {
				$stories_data = $connection->get_data();

				if ( isset( $stories_data ) ) {
					$header_data['stories'] = $stories_data;
					$this->set_header_data( $header_data );
				}
			} else {
				if ( $connection->is_wp_error() ) {
					SB_Instagram_API_Connect::handle_wp_remote_get_error( $connection->get_wp_error() );
				} else {
					SB_Instagram_API_Connect::handle_instagram_error( $connection->get_data(), $connected_accounts_for_feed[ $first_user ], 'header' );
				}
			}
		}
	}

	/**
	 * There isn't much info available for non-user feeds so just a basic
	 * amount of header data is set
	 *
	 * @param $feed_types_and_terms
	 */
	public function set_generic_header_data( $feed_types_and_terms ) {
		if ( isset( $feed_types_and_terms['hashtags_top'][0] ) || isset( $feed_types_and_terms['hashtags_recent'][0] ) ) {
			$term_and_params = isset( $feed_types_and_terms['hashtags_top'][0] ) ? $feed_types_and_terms['hashtags_top'][0] : $feed_types_and_terms['hashtags_recent'][0];
			$this->set_header_data( array( 'term' => $term_and_params['hashtag_name'] ) );
		}
	}

	/**
	 * Uses the settings to determine if avatars are going to be used.
	 * Can make feed creation faster if not.
	 *
	 * @param $settings
	 *
	 * @return bool
	 */
	public function need_avatars( $settings ) {
		if ( isset( $settings['type'] ) && $settings['type'] === 'hashtag' ) {
			return false;
		} elseif ( isset( $settings['disablelightbox'] ) && ($settings['disablelightbox'] === 'true' || $settings['disablelightbox'] === 'on') ) {
			return false;
		} else {
			return true;
		}
	}

	/**
	 * Available avatars are added to the feed as an attribute so they can be used in the lightbox
	 *
	 * @param $connected_accounts_in_feed
	 * @param $feed_types_and_terms
	 */
	public function set_up_feed_avatars( $connected_accounts_in_feed, $feed_types_and_terms ) {
		$header_data = $this->get_header_data();
		$ids_usernames_already_available = array();

		if ( ! empty( $header_data ) ) {
			if ( isset( $header_data['profile_picture_url'] ) ) {
				$this->set_avatar( $header_data['username'], $header_data['profile_picture_url'] );
				$ids_usernames_already_available[] = $header_data['id'];
				$ids_usernames_already_available[] = $header_data['username'];
			}
		}

		if ( isset( $feed_types_and_terms['users'] ) ) {
			foreach ( $feed_types_and_terms['users'] as $term_and_params ) {
				$user = $term_and_params['term'];

				if ( ! in_array( $user, $ids_usernames_already_available )
				     && ! in_array( $connected_accounts_in_feed[ $user ]['username'], $ids_usernames_already_available )
						&& isset( $connected_accounts_in_feed[ $user ]['profile_picture'] ) ) {
					$this->set_avatar( $connected_accounts_in_feed[ $user ]['username'], $connected_accounts_in_feed[ $user ]['profile_picture'] );

					$ids_usernames_already_available[] = $user;
					$ids_usernames_already_available[] = $connected_accounts_in_feed[ $user ]['username'];
				}
			}
		}

	}

	/**
	 * Creates a key value pair of the username and the url of
	 * the avatar image
	 *
	 * @param $name
	 * @param $url
	 */
	public function set_avatar( $name, $url ) {
		$this->username_avatars[ $name ] = $url;
	}

	/**
	 * @return array
	 */
	public function get_username_avatars() {
		return $this->username_avatars;
	}

	public function get_earliest_time_stamp( $feed_id ) {
		global $wpdb;

		$posts_table_name = $wpdb->prefix . SBI_INSTAGRAM_POSTS_TYPE;
		$feeds_posts_table_name = $wpdb->prefix . SBI_INSTAGRAM_FEEDS_POSTS;
		$feed_id_array = explode( '#', $feed_id );
		$feed_id = $feed_id_array[0];
		$is_top_post_feed = (substr( $feed_id, 4, 1 ) === '+');

		$timestamp_column = $is_top_post_feed ? 'top_time_stamp' : 'time_stamp';

		$results = $wpdb->get_col( $wpdb->prepare( "SELECT p.$timestamp_column FROM $posts_table_name AS p INNER JOIN $feeds_posts_table_name AS f ON p.id = f.id 
			WHERE BINARY f.feed_id = %s 
			ORDER BY p.$timestamp_column
			ASC LIMIT 1", $feed_id ) );

		$return = isset( $results[0] ) ? $results[0] : date( 'Y-m-d H:i:s' );

		return $return;
	}

	/**
	 * Used for filtering a single API request worth of posts
	 *
	 * @param $post_set
	 *
	 * @return mixed
	 *
	 * @since 5.0
	 */
	protected function filter_posts( $post_set, $settings = array() ) {

		if ( empty( $settings['includewords'] )
		     && empty( $settings['excludewords'] )
		     && empty( $settings['whitelist'] )
		     && empty( $settings['hidephotos'] )
		     && $settings['media'] === 'all') {
			return $post_set;
		}

		$includewords = ! empty( $settings['includewords'] ) ? explode( ',', $settings['includewords'] ) : array();
		$excludewords = ! empty( $settings['excludewords'] ) ? explode( ',', $settings['excludewords'] ) : array();
		$hide_photos = ! empty( $settings['hidephotos'] ) && empty( $settings['doingModerationMode'] ) ? explode( ',', str_replace( ' ', '', $settings['hidephotos'] ) ) : array();
		$white_list = ! empty( $settings['whitelist'] ) && empty( $settings['doingModerationMode'] ) ? get_option( 'sb_instagram_white_lists_'.$settings['whitelist'], array() ) : false;
		$media_filter = $settings['media'] !== 'all' ? $settings['media'] : false;

		$filtered_posts = array();
		foreach ( $post_set as $post ) {
			$padded_caption = ' ' . str_replace('+', ' ',  urlencode( str_replace( '#', ' HASHTAG', strtolower( SB_Instagram_Parse_Pro::get_caption( $post ) ) ) ) ) . ' ';
			$id = SB_Instagram_Parse_Pro::get_post_id( $post );

			$is_hidden = false;
			$passes_media_filter = true;
			if ( ! empty( $hide_photos )
			     && (in_array( $id, $hide_photos, true ) || in_array( 'sbi_' . $id, $hide_photos, true )) ) {
				$is_hidden = true;
			}

			if ( $media_filter ) {
				$media_type = SB_Instagram_Parse_Pro::get_media_type( $post );
				if ( $media_filter === 'videos' ) {
					if ( $media_type !== 'video' ) {
						$passes_media_filter = false;
					}
				} else {
					if ( $media_type === 'video' ) {
						$passes_media_filter = false;
					}
				}
			}

			// any blocked photos will not pass any additional filters so don't bother processing
			if ( ! $is_hidden && $passes_media_filter ) {
				$is_on_white_list = false;
				$has_includeword = false;
				$not_has_excludeword = false;

				if ( $white_list ) {
					if ( in_array( $id, $white_list, true ) || in_array( 'sbi_' . $id, $white_list, true ) ) {
						$is_on_white_list = true;
					}
				} elseif ( ! empty( $includewords ) ) {
					foreach ( $includewords as $includeword ) {
						$converted_includeword = str_replace('+', ' ', urlencode( str_replace( '#', 'HASHTAG', strtolower( $includeword ) ) ) );
						if ( preg_match('/\b'.$converted_includeword.'\b/i', $padded_caption, $matches ) ) {
							$has_includeword = true;
						}
					}
				} elseif ( ! empty( $excludewords ) ) {
					foreach ( $excludewords as $excludeword ) {
						$converted_excludeword = str_replace('+', ' ', urlencode( str_replace( '#', 'HASHTAG', strtolower( $excludeword ) ) ) );
						if ( ! preg_match('/\b'.$converted_excludeword.'\b/i', $padded_caption, $matches ) ) {
							$not_has_excludeword = true;
						}
					}
				} else {
					// no other filters so it belongs in the feed
					$filtered_posts[] = $post;
				}

				if ( $is_on_white_list || $has_includeword || $not_has_excludeword ) {
					$filtered_posts[] = $post;
				}
			}

		}

		return $filtered_posts;
	}

	protected function feed_is_complete( $settings, $offset = 0 ) {
		if ( ! empty( $settings['whitelist_ids'] ) ) {
			$total_posts_loaded = $settings['num'] + $offset;

			if ( (int)$settings['whitelist_num'] <= $total_posts_loaded ) {
				return true;
			}
		}

		return false;
	}

	protected function add_other_atts( $other_atts, $settings ) {
		$options_att_arr = array();

		$layout = $settings['layout'];
		if ( ! in_array( $layout, array( 'masonry', 'highlight', 'carousel' ) ) ) {
			$layout = 'grid';
		}

		if ( $layout === 'carousel' ) {
			$arrows = $settings['carouselarrows'] == 'true' || $settings['carouselarrows'] == 'on' || $settings['carouselarrows'] == 1 || $settings['carouselarrows'] == '1';
			$pag = $settings['carouselpag'] == 'true' || $settings['carouselpag'] == 'on' || $settings['carouselpag'] == 1 || $settings['carouselpag'] == '1';
			$autoplay = $settings['carouselautoplay'] == 'true' || $settings['carouselautoplay'] == 'on' || $settings['carouselautoplay'] == 1 || $settings['carouselautoplay'] == '1';
			$time = $autoplay ? (int)$settings['carouseltime'] : false;
			$loop = ! empty( $settings['carouselloop'] ) && ($settings['carouselloop'] !== 'rewind') ? false : true;
			$rows = ! empty( $settings['carouselrows'] ) ? min( (int)$settings['carouselrows'], 2 ) : 1;
			$options_att_arr['carousel'] = array( $arrows, $pag, $autoplay, $time, $loop, $rows );
		} elseif ( $layout === 'highlight' ) {
			$type = trim( $settings['highlighttype'] );
			$pattern = trim( $settings['highlightpattern'] );
			$offset = (int)trim( $settings['highlightoffset'] );
			$hashtag = str_replace( ',', '|', trim( str_replace( array( '#', ' '), '', $settings['highlighthashtag'] ) ) );
			$ids = str_replace( ',', '|', trim( str_replace( array( 'sbi_', ''), '', $settings['highlightids'] ) ) );
			$options_att_arr['highlight'] = array( $type, $pattern, $offset, $hashtag, $ids );
		} elseif ( $layout === 'masonry' ) {
			$options_att_arr['masonry'] = true;
		} else {
			$options_att_arr['grid'] = true;
		}
		$autoscroll = $settings['autoscroll'] == 'true' || $settings['autoscroll'] == 'on' || $settings['autoscroll'] == 1 || $settings['autoscroll'] == '1';

		if ( $autoscroll ) {
			$options_att_arr['autoscroll'] = max( 20, (int)$settings['autoscrolldistance'] );
		}

		$mediavine = $settings['mediavine'] == 'true' || $settings['mediavine'] == 'on' || $settings['mediavine'] === true;
		if ( $mediavine ) {
			$options_att_arr['mediavine'] = true;
		}

		if ( isset( $settings['feed_avatars'] ) ) {
			$options_att_arr['avatars'] = $settings['feed_avatars'];
		}

		$disablelightbox = $settings['disablelightbox'] == 'true' || $settings['disablelightbox'] == 'on' || $settings['disablelightbox'] == 1 || $settings['disablelightbox'] == '1';
		if ( $disablelightbox ) {
			$options_att_arr['disablelightbox'] = true;
		} else {
			$lightboxcomments = $settings['lightboxcomments'] == 'true' || $settings['lightboxcomments'] == 'on' || $settings['lightboxcomments'] == 1 || $settings['lightboxcomments'] == '1';
			if ( $lightboxcomments ) {
				$options_att_arr['lightboxcomments'] = max( 1, (int)$settings['numcomments'] );
			}
		}

		$captionlinks = $settings['captionlinks'] == 'true' || $settings['captionlinks'] == 'on' || $settings['captionlinks'] == 1 || $settings['captionlinks'] == '1';
		if ( $captionlinks ) {
			$options_att_arr['captionlinks'] = true;
		}

		$disable_mobile = $settings['disablemobile'];
		( $disable_mobile == 'on' || $disable_mobile == 'true' || $disable_mobile == true ) ? $disable_mobile = true : $disable_mobile = false;
		if( $settings[ 'disablemobile' ] === 'false' ) $disable_mobile = '';
		if ( $disable_mobile !== true && $settings['colsmobile'] !== 'same' ) {
			$colsmobile = (int)( $settings['colsmobile'] ) > 0 ? (int)$settings['colsmobile'] : 'auto';
		} else {
			$colsmobile = (int)( $settings['cols'] ) > 0 ? (int)$settings['cols'] : 4;
		}
		$options_att_arr['colsmobile'] = $colsmobile;

		//captionsize
		if ( ! empty( $settings['captionsize'] ) ) {
			$options_att_arr['captionsize'] = (int)$settings['captionsize'];
		}

		if ( ! empty( $settings['captionlength'] ) ) {
			$options_att_arr['captionlength'] = (int)$settings['captionlength'];
		}

		$moderation_mode = isset( $settings['doingModerationMode'] );
		if ( $moderation_mode ) {
			$mod_index = isset( $_GET['sbi_moderation_index'] ) ? sanitize_text_field( substr( $_GET['sbi_moderation_index'], 0, 10 ) ) : '0';
			$options_att_arr['modindex'] = esc_attr( $mod_index );


			if ( ! empty( $settings['whitelist'] ) ) {
				$white_list_name = $settings['whitelist'];
				$white_list_ids = ! empty( $settings['whitelist'] ) ? get_option( 'sb_instagram_white_lists_'.$settings['whitelist'], array() ) : false;
				$options_att_arr['whiteListName'] = $white_list_name;
				$options_att_arr['whiteListIDs'] = $white_list_ids;
			}


			$hide_photos = ! empty( $settings['hidephotos'] ) ? explode( ',', str_replace( ' ', '', $settings['hidephotos'] ) ) : array();
			if ( ! empty( $hide_photos ) ) {
				$options_att_arr['hidePhotos'] = $hide_photos;
			}
		}

		if ( $settings['addModerationModeLink'] ) {
			$options_att_arr['moderationLink'] = true;
		}

		$other_atts .= ' data-options="'.esc_attr( json_encode( $options_att_arr ) ).'"';

		return $other_atts;
	}


	protected function add_classes( $settings ) {
		$classes = array();

		$moderation_mode = (isset ( $_GET['sbi_moderation_mode'] ) && $_GET['sbi_moderation_mode'] === 'true' && current_user_can( 'edit_posts' ));

		if ( $moderation_mode ) {
			$classes[] = 'sbi_moderation_mode';
		}
		return array();
	}

	/**
	 * When the feed is loaded with AJAX, the JavaScript for the plugin
	 * needs to be triggered again. This function is a workaround that adds
	 * the file and settings to the page whenever the feed is generated.
	 *
	 * @return string
	 *
	 * @since 5.0
	 */
	public static function get_ajax_page_load_html() {
		$sbi_options = sbi_get_database_settings();
		$font_method = isset( $sbi_options['sbi_font_method'] ) ? $sbi_options['sbi_font_method'] : 'svg';
		$upload = wp_upload_dir();
		$resized_url = trailingslashit( $upload['baseurl'] ) . trailingslashit( SBI_UPLOADS_NAME );
		$br_adjust = isset( $sbi_options['sbi_br_adjust'] ) && ($sbi_options['sbi_br_adjust'] == 'false' || $sbi_options['sbi_br_adjust'] == '0' || $sbi_options['sbi_br_adjust'] == false) ? false : true;

		$js_options = array(
			'font_method' => $font_method,
			'placeholder' => trailingslashit( SBI_PLUGIN_URL ) . 'img/placeholder.png',
			'resized_url' => $resized_url,
			'br_adjust' => $br_adjust
		);

		$encoded_options = json_encode( $js_options );

		$js_option_html = '<script type="text/javascript">var sb_instagram_js_options = ' . $encoded_options . ';</script>';
		$js_option_html .= "<script type='text/javascript' src='" . trailingslashit( SBI_PLUGIN_URL ) . 'js/sb-instagram.js?ver=' . SBIVER . "'></script>";

		return $js_option_html;
	}
}